export const onTokenizeSuccessMixin = {
  props: {
    tokenizeSuccessStatus: {
      type: [Boolean, Error],
      required: false
    },

    tokenizeSuccess: {
      type: Function,
      required: false
    }
  },

  data() {
    return {
      onTokenizeSuccessPromise: null,
    }
  },

  watch: {
    tokenizeSuccessStatus(tokenizeSuccessStatus) {
      if (!this.isWaitingForOnTokenizeSuccess()) {
        return
      }

      if (tokenizeSuccessStatus === true) {
        this.onTokenizeSuccessPromise.resolve()
        this.clearOnTokenizeSuccess()
      } else if (tokenizeSuccessStatus instanceof Error) {
        this.onTokenizeSuccessPromise.reject(tokenizeSuccessStatus)
        this.clearOnTokenizeSuccess()
      }
    }
  },

  methods: {
    isWaitingForOnTokenizeSuccess() {
      return !!this.onTokenizeSuccessPromise
    },

    resolveOnTokenizeSuccess(err) {
      if (!this.isWaitingForOnTokenizeSuccess()) {
        return
      }

      if (err === null || err === undefined) {
        this.onTokenizeSuccessPromise.resolve()
      } else if (err instanceof Error) {
        this.onTokenizeSuccessPromise.reject(err)
      }

      this.clearOnTokenizeSuccess()
    },

    clearOnTokenizeSuccess() {
      this.onTokenizeSuccessPromise = null
      this.$emit('tokenize-success-finish')
    },

    onTokenizeSuccess(...args) {
      // Using function props
      if (this.tokenizeSuccess) {
        return this.tokenizeSuccess(...args)
      }

      // Promise to return
      const promise = new Promise((resolve, reject) => {
        this.onTokenizeSuccessPromise = {resolve, reject}
      })

      // Create callback to pass to event
      const next = err => {
        this.resolveOnTokenizeSuccess(err)
      }

      this.$emit('tokenize-success', ...args, next)

      return promise
    }

  }

}
